import Component from '@glimmer/component';
import { action } from '@ember/object';
import { inject as service } from '@ember/service';
import { tracked } from '@glimmer/tracking';

export default class FormsLoginComponent extends Component {
  @service() session;
  @service() router;
  @service() request;
  @tracked isLoading = false;

  @tracked errorMessage;
  @tracked email;
  @tracked password;
  @tracked otp;
  @tracked otpType;
  @tracked showingOtpForm = false;

  get authenticatedRedirectLocation() {
    return this.args.authenticatedRedirectLocation ?? 'unit.dashboard.index.electric';
  }

  handleLoginError(error) {
    if (typeof error === 'string') {
      this.errorMessage = error;
    } else if (Array.isArray(error) && error.length > 0 && typeof error[0]) {
      this.errorMessage = error[0];
    } else {
      this.errorMessage = 'Login failed. Please try again.';
    }
  }

  @action
  async authenticate(event) {
    event.preventDefault();
    this.errorMessage = null;
    this.isLoading = true;
    try {
      await this.session.authenticate(
        'authenticator:token',
        this.email,
        this.password,
        this.otp
      );
    } catch (response) {
      if (Object.keys(response).includes('otp')) {
        this.otpType = response.otp;
        this.showingOtpForm = true;
      } else {
        this.handleLoginError(response.errors);
      }
    } finally {
      this.isLoading = false;
    }

    if (this.session.isAuthenticated) {
      this.router.transitionTo(this.authenticatedRedirectLocation);
    }
  }

  @action
  async getNewCode() {
    this.request.request(
      'refresh_otp',
      {
        email: this.email,
        password: this.password,
      },
      false,
      false
    );
  }
}
