import classic from 'ember-classic-decorator';
import { getOwner } from '@ember/application';
import Service, { inject as service } from '@ember/service';
import fetch from 'fetch';
import { task } from 'ember-concurrency';
import fastbootSafeUrlSearchParams from 'sli-ember-library/utils/fastboot-safe-url-search-params';

// TODO We should phase this out. Try to use ember-data and json-api resource
// for all requests
@classic
export default class RequestService extends Service {
  @service session;

  request(url, data, hasNamespace = false, hasHost = false, method = 'POST', headers) {
    let token = this.session.data.authenticated.access_token;

    url = `${hasHost ? "" : getOwner(this).resolveRegistration('config:environment').hostUrl}${hasNamespace ? `/${getOwner(this).resolveRegistration('config:environment').namespace}` : ''}${hasHost ? '' : '/'}${url}`;

    headers = headers || {
      'Accept': 'application/json',
      'Content-Type': 'application/json'
    };

    if (token && !hasHost) {
      headers['Authorization'] = `Bearer ${token}`;
    }

    let options = {
      method,
      crossDomain: true,
      headers
    }

    if (data) {
      options.body = JSON.stringify(data);
    }

    return fetch(url, options);
  }

  @task
  historic = function* (relativeUrl, range, aggregate = null, aggregateType = 'minute') {
    const params = aggregate ? {
      range,
      aggregate,
      aggregateType
    } : { range };
    const url = `${relativeUrl}?${fastbootSafeUrlSearchParams(params)}`;
    const method = 'GET';

    const response = yield this.request(
      url,
      null,
      true,
      false,
      method
    );

    const data = yield response.json();
    return yield data && response.status === 200
      ? { data, url, method }
      : {};
  }
}
