# Directory Structure: Packages

- Client
- Composites
- Supporting

## Client

Each consumer application built for the business purpose is a client into our architecture. The Tenant and Admin application are good examples of clients that consume design system and other business packages to build up ambitious experiences through digital services.

## Composites

Each composite has its own directory and package. All packages belonging to a composite are a direct descendant of the respective composite directory.

This `connect-admin-ui/` is part of the Composite Architecture. It contains all composites the frontend is embraced of. Each composite consists of one or more `pieces`. Every constituent maps to one or more packages contained in the respective composite directory.

```
connect-admin-ui/           -> @sli-admin-ui - Admin application where all composites integrate into
connect-tenant-ui/          -> @sli-tentant-ui -  Customer application where all composites integrate into
  sli-composites/           -> Addons spread in the github
    ...
    sli-customers/          -> @sli-customers
      addon/
        customer/           -> Internal shared utilities and business logic that may only be used inside the composite.
      ...
    sli-products/           -> @sli-products
      addon/
        products/           -> Internal shared utilities and business logic that may only be used inside the composite.
      ...
    sli-ember-simple-auth/  -> @sli-ember-simple-auth
      addon/                -> implements SLI authentication/authorization for Ember.js applications.
      index.js
```

The package name used in the package.json always follows this structure: `@sli-{composite}`.

Where {composite} is the name of the composite & matches the composite directory name. For instance would produce `@sli-customers` as the package name.

## Supporting

Supporting packages are considered public to all other packages. They contain generic code that is not strictly tied to a single composite/business function. For instance, this is where generic UI components, like form elements, modals, etc. belong.

```
sli-tower/           -> @sli-ui/tower - Package for our design system
  addon/
    components/
      /tower         -> namespace for design system components "<Tower::Button>"
        /modal
        /button
        /accordion   
    helpers/         -> public helpers, like "format-money"

sli-build/           -> @sli-build/brand-config - Package to support different brands/locales
  addon/
  index.js    
```

Packages will follow this naming scheme: `@sli-{topic}/{name}` 

