# Namespacing

- Packages
- Container-Resolvable Ember Entities
  - Components
  - Template Helpers
  - Services


## Packages

To not conflict with each other, all packages are namespaced with a [package scope](https://docs.npmjs.com/cli/v7/using-npm/scope). For more details, please refer to the [directory structure](./directory-structure.md)

## Container-Resolvable Ember Entities

Many Ember entities share a global namespace by default. This includes components, template helpers, services and some more. We namespace these by their composite name.

In the future primitives like template imports will allow us to develop an even better structure.

Primarily this rule is designed for public Ember addons, as they are consumed by other packages and get merged with their namespaces. If a package is strictly composite-private and is only consumed by engines (which by nature have their own namespaces), this rule may be ignored. If however the package is consumed by a public addon, the rule must be adhered, since the namespaces get merged. In doubt, it's safest to always apply this rule.

Example:

```
sli-customers/              -> The package name
  customer/                 -> The composite name
    addon/
      components/
        customer/           -> Same as the composite name
          registration/     -> <Customer::Registration />
            component.ts
            styles.css
            template.hbs
      helpers/
        customer/
          is-prospect.ts    -> {{customer/is-prospect}}
      services/
        customer/
          account.ts        -> @service('customer/account')
    app/                    -> re-exports to be merged with the host app tree
```

### Components

For components the story is a bit different, since every component is global by default. For that, namespace components by composite.

Schema

```hbs
<{composite}::{name} />
```

Example
```hbs
<Customer::Registration />
<Customer::RegistrationPage />
<Customer::DetailsPage />
```


### Template Helpers

Schema

```hbs
{{<composite>/<name>}}
```

Example

```hbs
{{customer/is-admin this.customer}}
```

### Services

Schema

```ts
@service('{composite}/{name}')
```

Example

```ts
import AccountService from '@sli-customers/customer/services/customer/account';

class Foo {
  @service('customer/account') account!: AccountService;
}
```
